/**
 * @copyright nhcarrigan
 * @license Naomi's Public License
 * @author Naomi Carrigan
 */
import { Client, Events, GatewayIntentBits } from "discord.js";
import { scheduleJob } from "node-schedule";
import { channels } from "./config/channels.js";
import { standup } from "./modules/standup.js";
import { logger } from "./utils/logger.js";

const activeIds: Array<string> = [];

process.on("unhandledRejection", (error) => {
  if (error instanceof Error) {
    void logger.error("Unhandled Rejection", error);
    return;
  }
  void logger.error("unhandled rejection", new Error(String(error)));
});

process.on("uncaughtException", (error) => {
  if (error instanceof Error) {
    void logger.error("Uncaught Exception", error);
    return;
  }
  void logger.error("uncaught exception", new Error(String(error)));
});

const client = new Client({
  intents: [ GatewayIntentBits.Guilds, GatewayIntentBits.GuildMessages ],
});

client.on(Events.ClientReady, () => {
  void logger.log("debug", "Bot is ready.");
  scheduleJob("reminders", "0 9 * * 1-5", async() => {
    await standup(client, activeIds);
  });
});

client.on(Events.MessageCreate, (message) => {
  if (!message.channel.isThread() || !message.inGuild()) {
    return;
  }
  if (message.channel.parentId === null) {
    return;
  }
  const isInTeamChannel = channels.some((channel) => {
    return channel.channelId === message.channel.parentId;
  });
  if (!isInTeamChannel) {
    return;
  }
  if (message.author.bot) {
    return;
  }
  if (!activeIds.includes(message.author.id)) {
    activeIds.push(message.author.id);
  }
});

await client.login(process.env.DISCORD_TOKEN);
